/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using Gtk;
using Gdk;
using MovieDatabase;
using System.Collections;
using Util;

public class DataView : Gtk.TreeView
{
    /* This structure is used to speed up the operations with filling TreeView with
     * data. */
    private class FillMovieInfo {
        
        public string BorrowedColor;
        public string OverBorrowedColor;
        public string NewColor;
        
        public Int64 OverBorrowedTS;
        public Int64 NewTS;

        /* Constructor */
        public FillMovieInfo()
        {
            Util.Debug.Message("Monotheka.exe", "FillMovieInfo", "Constructor...");

            // Colors
            BorrowedColor = ApplicationSettings.BorrowedColor;
            OverBorrowedColor = ApplicationSettings.OverBorrowedColor; 
            NewColor = ApplicationSettings.NewColor; 

            // Constants for * operation
            Int64 __Second = 10000000;
            Int64 __Day = 3600 * 24;

            // Days 
            Int64 __DaysOverBorrowed = (Int64) ApplicationSettings.DaysOverBorrowed;
            Int64 __DaysNew = (Int64) ApplicationSettings.DaysNew;   
            
            // Calculate            
            OverBorrowedTS = DateTime.Now.ToFileTime() - (__Second * __Day * __DaysOverBorrowed);
            NewTS = DateTime.Now.ToFileTime() - (__Second * __Day * __DaysNew);
        }


    }
    

    private SearchRequest LastSearch = null;
    private ColumnsRequest DBColumns;
    private ArrayList DBFields = new ArrayList();
    private int SortColumnId = -1;
    private SortType SortColumnMode;

    public Gtk.TreeStore Store; 
    public TreeIter SelectedIter;
        
    public event SelectedMovieHandler MovieSelected;
    public event SelectedMovieHandler MovieLeaved;

    public delegate void SelectedMovieHandler(object sender, string ID);

    private FillMovieInfo CurrentFillMovieInfo;

    //////////////////////////////////////////////////
    
    /* GConf keys */
    private const string GConf_Columns = "/apps/monotheka/mainwindow/dbcolumns" ;

    public void FillRowWithMovie(Movie _M)
    {
        if (SelectedIter.Stamp != 0) FillRowWithMovie(SelectedIter, _M);
        
    }

    public void FillRowWithMovie(TreeIter _TI, Movie _M)
    {
        if (CurrentFillMovieInfo == null) FillRowWithMovie(_TI,_M, new FillMovieInfo());
        else FillRowWithMovie(_TI,_M, CurrentFillMovieInfo);
    }


    private void FillRowWithMovie(TreeIter _TI, Movie _M, FillMovieInfo _FillInf)
    {
        int Status = 0; // 0 - normal, 1 - new, 2 - borrowed, 3 - overborrowed

        // Let's see status
        if (_M.Data[(int)MovieField.BORROWER] != "" && _M.Data[(int)MovieField.BORROWER] != null && Convert.ToInt64(_M.Data[(int)MovieField.BORROWEDDATE]) < _FillInf.OverBorrowedTS) Status = 3;
        else if (_M.Data[(int)MovieField.BORROWER] != "" && _M.Data[(int)MovieField.BORROWER] != null) Status = 2;
        else if (Convert.ToInt64(_M.Data[(int)MovieField.ADDEDDATE]) > _FillInf.NewTS) Status = 1;
        
        
        // Fill
        int __i=0;
        foreach (string __s in _M.Data) {
            if (__s != null) {
                
                if (((DataField)MovieDB.Fields[__i]).Type == DataFieldType.PIXBUF) {
                    if (__s != "") {
                        string[] __Flags_str = __s.Split(',');
                        MovieFlag[] __Flags = new MovieFlag[__Flags_str.Length];
                        
                        int __k = 0;
                        foreach (string __fl in __Flags_str) {
                            __Flags[__k] = (MovieFlag) Convert.ToInt32(__fl);
                            __k++;
                        }
                        
                        Store.SetValue(_TI, __i, FlagFactory.GetPixbufForFlags(__Flags));
                    } else {
                        Store.SetValue(_TI, __i, FlagFactory.IconSet[0]);
                        // TODO: Fixme, PIXBUF factory
                    }

                } else { 
                    string __markup;
                    
                    if (__i == (int) MovieField.ADDEDDATE || __i == (int) MovieField.BORROWEDDATE)
                        if (__s != null && __s != String.Empty)  __markup = DateTime.FromFileTime(Convert.ToInt64(__s)).ToLocalTime().ToString("d"); 
                        else __markup = String.Empty;
                    else __markup = Util.Strings.MarkupizeText(__s);

                    switch (Status) {
                    case 0:
                        break;
                    case 1:
                        __markup = "<span color=\"" + _FillInf.NewColor + "\">" + __markup + "</span>";
                        break;
                    case 2:
                        __markup = "<span color=\"" + _FillInf.BorrowedColor + "\">" + __markup + "</span>";
                        break;
                    case 3:
                        __markup = "<span color=\"" + _FillInf.OverBorrowedColor + "\">" + __markup + "</span>";
                        break;
                    }

                    // We don't want to set markup for ID, etc.
                    if (__i != (int) MovieField.ID) Store.SetValue(_TI, __i, __markup);
                    else Store.SetValue(_TI, __i, __s);

                    __markup = null;
                }
                
            }
            __i++;
        }

    }
    
    public void ShowColumn(MovieField _MF)
    {
        // Get old sorting
        int __OldSortColumn;
        SortType __OldSortType;
        Store.GetSortColumnId(out __OldSortColumn, out __OldSortType);
        
        // Reset
        Store.SetSortColumnId(0, SortType.Ascending);
        DBColumns += _MF;
        Columns[(int)_MF].Visible = true;

        ColumnsRequest __NewColumn = new ColumnsRequest(_MF);
        MovieDB.ListDatabase(__NewColumn, LastSearch,new MovieDB.ListDatabase_ProcessRecord(UpdateTreeView)); 

        Store.SetSortColumnId(__OldSortColumn, __OldSortType);
        
        GConfRegistry.Set (GConf_Columns, DBColumns.ToString());
    }

    
    public void HideColumn(MovieField _MF)
    {
        DBColumns -= _MF;
        Columns[(int)_MF].Visible = false;
        
        GConfRegistry.Set (GConf_Columns, DBColumns.ToString());
    }



    /* Callback function, called by ListDatabase / RefreshView */
    public object AddToTreeView (Movie _M, object _LastObject)
    {
        // _LastObject is not really used here
        TreeIter __lastrow = Store.AppendValues();
        FillRowWithMovie(__lastrow, _M);
        return __lastrow;
    }

    public bool RemoveFromTreeView(TreeIter _TI)
    {
        Store.Remove(ref _TI);
        
        if (_TI.Stamp != 0) { 
            SetSelection(_TI);
            return true;
        }  else {
            SelectedIter.Stamp = 0;
            return false;
        }
    }

    public void SetSelection(TreeIter _TI)
    {
        TreePath __TP = Store.GetPath(_TI);
        SetCursor(__TP, Columns[0], false);
    }

    public void SetSelection()
    {
        TreeIter __TI;
        Store.GetIterFirst(out __TI);
        SetSelection(__TI);
    }


    public object UpdateTreeView (Movie _M, object _LastObject)
    {
        Gtk.TreeIter __LastIter;
        if (_LastObject == null) Store.GetIterFirst(out __LastIter);
        else { 
            __LastIter = (Gtk.TreeIter) _LastObject;
            if (!Store.IterNext(ref __LastIter)) throw new Exception("Fatal error! DataView & DB not syncing!");
        }
                
        FillRowWithMovie(__LastIter, _M);
        return __LastIter;
    }
        
    
        
    /* Constructor */
    public DataView(ColumnsRequest _CR) : base ()
    {
        
        Store = new Gtk.TreeStore	(typeof (string), typeof (Pixbuf), typeof (string), typeof (string),
                                         typeof (string), typeof (string), typeof (string), typeof (string),
                                         typeof (string), typeof (string), typeof (string), typeof (string),
                                         typeof (string), typeof (string), typeof (string), typeof (string),
                                         typeof (string), typeof (string), typeof (string), typeof (string),
                                         typeof (string));
        
        Model = Store;
        HeadersVisible = true;
        HeadersClickable = true;
        RulesHint = true;

        DBColumns = _CR;
        DBFields = MovieDB.Fields;

        // This is to surpress some error msg
        Store.SetDefaultSortFunc(new TreeIterCompareFunc(NullCompareFunc) , (IntPtr) null, null);

        int __i = 0;
        foreach (DataField Field in MovieDB.Fields) {
            
            // Create column 
            TreeViewColumn __clmn;
            if (Field.Type == DataFieldType.NUMBER) {
                __clmn = AppendColumn (Field.Name, new CellRendererText (), "markup", __i);
                Store.SetSortFunc(__i, new TreeIterCompareFunc(NumericalCompareFunc) , (IntPtr) null, null);
            } else if (Field.Type == DataFieldType.TEXT){
                __clmn = AppendColumn (Field.Name, new CellRendererText (), "markup", __i);
                Store.SetSortFunc(__i, new TreeIterCompareFunc(TextualCompareFunc) , (IntPtr) null, null);
            } else if (Field.Type == DataFieldType.TIMESTAMP) {
                __clmn = AppendColumn (Field.Name, new CellRendererText (), "markup", __i);
                Store.SetSortFunc(__i, new TreeIterCompareFunc(TimestampCompareFunc) , (IntPtr) null, null);
            } else {
                __clmn = AppendColumn (Field.Name, new CellRendererPixbuf (), "pixbuf", __i);
                Store.SetSortFunc(__i, new TreeIterCompareFunc(NullCompareFunc) , (IntPtr) null, null);
            }
            
 
            if (_CR.Columns[__i] == false || Field.Name == "ID") __clmn.Visible = false;
            
            __clmn.Resizable = true;

            if (Field.Type != DataFieldType.PIXBUF) __clmn.Clickable = true;
            __clmn.Clickable = false;

            __clmn.SortColumnId = __i;
            __i++;
        }
        
   

    }


    public int NullCompareFunc (TreeModel model, TreeIter a, TreeIter b)
    {
        int __aval, __bval;
        
        try {
            __aval = Convert.ToInt32(Store.GetValue(a, 0));
        } catch {
            __aval = 0;
        }
        try {
            __bval = Convert.ToInt32(Store.GetValue(b, 0));
        } catch {
            __bval = 0;
        }

        int ret = 0;
        
        if (__aval > __bval) ret = 1; 
        else if (__aval < __bval) ret = -1; 
        return ret;

    }


    public int TimestampCompareFunc (TreeModel model, TreeIter a, TreeIter b)
    {
        int Id;
        SortType _S;
        
        // TODO: Big shit below
        Store.GetSortColumnId(out Id, out _S);
        // TODO: Some optimizations here!
        DateTime __aval, __bval;
        
        try {
            __aval = DateTime.Parse(Util.Strings.StripMarkupFaster((string) Store.GetValue(a, Id)));
        } catch {
            __aval = DateTime.Parse("1950-01-01");
        }
        try {
            __bval = DateTime.Parse(Util.Strings.StripMarkupFaster((string) Store.GetValue(b, Id)));
        } catch {
            __bval = DateTime.Parse("1950-01-01");
        }

        int ret = 0;
        
        if (__aval > __bval) ret = 1; 
        else if (__aval < __bval) ret = -1; 
        return ret;
    }

    
    public int NumericalCompareFunc (TreeModel model, TreeIter a, TreeIter b)
    {
        int Id;
        SortType _S;
        
        // TODO: Big shit below
        Store.GetSortColumnId(out Id, out _S);
        // TODO: Some optimizations here!
        int __aval, __bval;
        
        try {
            __aval = Convert.ToInt32(Util.Strings.StripMarkupFaster((string) Store.GetValue(a, Id)));
        } catch {
            __aval = 0;
        }
        try {
            __bval = Convert.ToInt32(Util.Strings.StripMarkupFaster((string) Store.GetValue(b, Id)));
        } catch {
            __bval = 0;
        }

        int ret = 0;
        
        if (__aval > __bval) ret = 1; 
        else if (__aval < __bval) ret = -1; 
        return ret;
    }

    
    public int TextualCompareFunc (TreeModel model, TreeIter a, TreeIter b)
    {
        int Id;
        SortType _S;
        
        // TODO: Big shit below
        Store.GetSortColumnId(out Id, out _S);
        // TODO: Some optimizations here!
        string __aval, __bval;
        __aval = Util.Strings.StripMarkupFaster ((string) Store.GetValue(a, Id));
        __bval = Util.Strings.StripMarkupFaster ((string) Store.GetValue(b, Id));
        return String.Compare(__aval, __bval, true);
    }


    public int RefreshView(bool _KeepSelection) 
    {
        // Get selection
        TreePath __Current = null;
        // FIXME: Check it ITER valid
        if (_KeepSelection) __Current = Store.GetPath(SelectedIter);
        
        Store.Clear();
        CurrentFillMovieInfo = new FillMovieInfo();
        int __Rows = MovieDB.ListDatabase(DBColumns,LastSearch,new MovieDB.ListDatabase_ProcessRecord(AddToTreeView)); 
        CurrentFillMovieInfo = null;

        // Restore Selection
        if (_KeepSelection && __Current != null) SetCursor(__Current, Columns[0], false);
        
        return __Rows;
    }

    public int RefreshView(SearchRequest _SR)
    {
        LastSearch = _SR;
        return RefreshView(false);
        
    }

    protected override void OnCursorChanged () 
    {
        if (SelectedIter.Stamp != 0 && MovieLeaved != null) MovieLeaved(this, GetSelectedId());

        // TODO: Fi this? 
        TreeModel model = Store;
        TreeSelection clicked = Selection;
        clicked.GetSelected(out model , out SelectedIter);
        if (MovieSelected != null) MovieSelected(this, GetSelectedId());
    }


 
    private void ChangeSortColumn(int _SortClmn)
    {        
        Console.WriteLine("Changing sort column");
        if (SortColumnId == -1) {
                    Columns[_SortClmn].SortIndicator = true;
                    
                    SortColumnId = _SortClmn;
                    SortColumnMode = SortType.Ascending;
                    
                    Store.SetSortColumnId(_SortClmn, SortType.Ascending);
                    Columns[_SortClmn].SortOrder = SortType.Ascending;
        } else
            if (_SortClmn == SortColumnId) {
                
                // Same column, order change
                if (SortColumnMode == SortType.Ascending) {
                    Columns[_SortClmn].SortOrder = SortType.Descending;
                    SortColumnMode = SortType.Descending;
                    Store.SetSortColumnId(_SortClmn, SortType.Descending);
                    
                }
                else {
                    Columns[_SortClmn].SortOrder = SortType.Ascending;
                    SortColumnMode = SortType.Ascending;
                    Store.SetSortColumnId(_SortClmn, SortType.Ascending);
                    
                }
            }
            else {
                // Other column 
                // Clear indicator
                Columns[SortColumnId].SortIndicator = false;
                // Change
                Columns[_SortClmn].SortIndicator = true;
                Columns[_SortClmn].SortOrder = SortType.Ascending;
                SortColumnId = _SortClmn;
                SortColumnMode = SortType.Ascending;
                
                Store.SetSortColumnId(_SortClmn, SortType.Ascending);

            }
    }

    

    public string GetSelectedId()
    {
        return ((string) Store.GetValue(SelectedIter, 0));
    }
                
                    


    
    public void HeaderClicked(object sender, EventArgs e)
    {
        //TreeViewColumn __clmn = (TreeViewColumn) sender;
        //Console.WriteLine(__clmn.SortColumnId);
        //if (__clmn.SortColumnId != 1) ChangeSortColumn(__clmn.SortColumnId);
        
    }



}
