/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using Gtk;
using System;
using MovieDatabase;
using System.Collections;

public class MovieView : HTML
{
    // Translatable strings
    private static readonly string Msg_BorrowedDaysAgo = "Borrowed {0} days ago";
    private static readonly string Msg_BorrowedDayAgo = "Borrowed one day ago";
    private static readonly string Msg_JustBorrowed = "Borrowed a few moments ago";
    private static readonly string Msg_OverBorrowed = "Borrowed {0} days ago, over-borrowed!";
    
    // Media images
    Gtk.Image Disc1_GtkImage;
    Gtk.Image Disc2_GtkImage;
    Gtk.Image Disc3_GtkImage;
    Gtk.Image Disc4_GtkImage;
    Gtk.Label Title_GtkLabel;
    Gtk.Label BorrowedFor_GtkLabel;

    public Movie DisplayedMovie = null;
    public bool MovieModified = false;
    
    // Constructor
    public MovieView(Style P, Image _1, Image _2, Image _3, Image _4, Label _Title, Label _BorrowedFor) : base()
    {
        Disc1_GtkImage = _1;
        Disc2_GtkImage = _2;
        Disc3_GtkImage = _3;
        Disc4_GtkImage = _4;
        Title_GtkLabel = _Title;
        BorrowedFor_GtkLabel = _BorrowedFor;

        MagicLinks = true;

        // Set background color to default GTKRC
        // FIXME: Some more logic here, we don't need external style
        ModifyBase(StateType.Normal, P.Backgrounds[0]);
        Reset();
    }

    // Display data about a given movie
    
    private string GetFieldName(MovieField _F)
    {
        DataField __DF = (DataField) MovieDB.Fields[(int) _F];
        return __DF.Name;
    }

     
    public void Reset()
    {
        LoadFromString("<html></html>");
        DisplayedMovie = null;

        Disc4_GtkImage.SetFromStock("gnome-empty", IconSize.Dialog);
        Disc3_GtkImage.SetFromStock("gnome-empty", IconSize.Dialog);
        Disc2_GtkImage.SetFromStock("gnome-empty", IconSize.Dialog);
        Disc1_GtkImage.SetFromStock("gnome-empty", IconSize.Dialog);
        Title_GtkLabel.Markup = "";
        BorrowedFor_GtkLabel.Markup = "";
    }


    public void Refresh()
    {
        if (DisplayedMovie != null) RenderData();
    }

    public void LoadMovie(Movie _M)
    {
        MovieModified = false;
        DisplayedMovie = _M;
        RenderData();
    }

    public void RenderData()
    {
        // FIXME: We don't need _M
        Movie _M = DisplayedMovie;
        // Set the gtk images
        string __MediaStockIcon = "gnome-dev-cdrom";

        // Dvd as an option...
        if ( _M.Data[(int)MovieField.VIDEOFORMAT] != null && _M.Data[(int)MovieField.VIDEOFORMAT] != "")
            if (_M.Data[(int)MovieField.VIDEOFORMAT].IndexOf("DVD") != -1 || _M.Data[(int)MovieField.VIDEOFORMAT].IndexOf("dvd") != -1) __MediaStockIcon = "gnome-dev-dvd";
        

		
        int __Discs = 0;
        try {
            // FIXME: != String.Empty;
            if (_M.Data[(int)MovieField.DISCS] != null) __Discs = Convert.ToInt32(_M.Data[(int)MovieField.DISCS]);
        }
        catch {
            __Discs = 0;
        }

        // Calculate timespans!
        int __DaysBorrowed = -1;
        try {
            if (_M.Data[(int)MovieField.BORROWEDDATE] != null && _M.Data[(int)MovieField.BORROWEDDATE] != String.Empty) {
                // Let's calculate how many days it's borrowed
                Int64 __BorrowedOnTicks = Convert.ToInt64(_M.Data[(int)MovieField.BORROWEDDATE]);
                DateTime __BorrowedOn = new DateTime(__BorrowedOnTicks);
                // FIXME: 584388 ???
                __DaysBorrowed = (DateTime.Now.ToLocalTime() - __BorrowedOn).Days - 584388;
            }
        } catch {
            __DaysBorrowed = -1;
        }

       
        // Setting the pixbufs shit
        if ( __Discs > 3) Disc4_GtkImage.SetFromStock(__MediaStockIcon, IconSize.Dialog);
        else Disc4_GtkImage.SetFromStock("gnome-empty", IconSize.Dialog);
        if ( __Discs > 2) Disc3_GtkImage.SetFromStock(__MediaStockIcon, IconSize.Dialog);
        else Disc3_GtkImage.SetFromStock("gnome-empty", IconSize.Dialog);
        if ( __Discs > 1) Disc2_GtkImage.SetFromStock(__MediaStockIcon, IconSize.Dialog);
        else Disc2_GtkImage.SetFromStock("gnome-empty", IconSize.Dialog);
        if ( __Discs > 0) Disc1_GtkImage.SetFromStock(__MediaStockIcon, IconSize.Dialog);
        else Disc1_GtkImage.SetFromStock("gnome-empty", IconSize.Dialog);

        // Set the title
        if (_M.Data[(int)MovieField.TITLE] != null && _M.Data[(int)MovieField.TITLE] != "") Title_GtkLabel.Markup = "<big><big><b><i>" + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.TITLE]) + "</i></b></big></big>\n<i>" + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.TRANSLATEDTITLE]) + "</i>";
        else Title_GtkLabel.Markup = "<big><big><b><i>(No title)</i></b></big></big>";

        // Set the Borrowed string
        switch (__DaysBorrowed) {
            case 1:
                BorrowedFor_GtkLabel.Markup = String.Format("<span color=\"{0}\">{1}</span>", ApplicationSettings.BorrowedColor, Msg_BorrowedDayAgo); 
                break;
            case 0:
                BorrowedFor_GtkLabel.Markup = String.Format("<span color=\"{0}\">{1}</span>", ApplicationSettings.BorrowedColor, Msg_JustBorrowed); 
                break;
            case -1:
                BorrowedFor_GtkLabel.Markup = "";
                break;
            default:
                if (__DaysBorrowed <= ApplicationSettings.DaysOverBorrowed) BorrowedFor_GtkLabel.Markup = String.Format("<span color=\"{0}\">{1}</span>", ApplicationSettings.BorrowedColor, String.Format(Msg_BorrowedDaysAgo, __DaysBorrowed)); 
                else BorrowedFor_GtkLabel.Markup = String.Format("<span color=\"{0}\">{1}</span>", ApplicationSettings.OverBorrowedColor, String.Format(Msg_OverBorrowed, __DaysBorrowed)); 
                break; 
            } 

        string __NewText = "<html><body leftmargin=0 topmargin=0>";
        
        // NO
        if (_M.Data[(int)MovieField.NO] != "") __NewText += "<b>" + GetFieldName(MovieField.NO) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.NO]) + "<br>";
        
        // Director
        if (_M.Data[(int)MovieField.DIRECTOR] != "") __NewText += "<b>" + GetFieldName(MovieField.DIRECTOR) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.DIRECTOR]) + "<br>";
        
        // Time
        if (_M.Data[(int)MovieField.TIME] != "") __NewText += "<b>" + GetFieldName(MovieField.TIME) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.TIME]) + "min" + "<br>";
        
        // Year
        if (_M.Data[(int)MovieField.YEAR] != "") __NewText += "<b>" + GetFieldName(MovieField.YEAR) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.YEAR]) + "<br>";


        // Country
        if (_M.Data[(int)MovieField.COUNTRY] != "") __NewText += "<b>" + GetFieldName(MovieField.COUNTRY) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.COUNTRY]) + "<br>";

        // GENRE
        if (_M.Data[(int)MovieField.GENRE] != "") __NewText += "<b>" + GetFieldName(MovieField.GENRE) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.GENRE]) + "<br>";

        __NewText += "<br>";
        
        /*
        // Discs
        if (_M.Data[(int)MovieField.DISCS] != "") __NewText += "<b>" + GetFieldName(MovieField.DISCS) + ":</b> " + _M.Data[(int)MovieField.DISCS] + "<br>";
        */
        
        // VideoFormat
        if (_M.Data[(int)MovieField.VIDEOFORMAT] != "") __NewText += "<b>" + GetFieldName(MovieField.VIDEOFORMAT) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.VIDEOFORMAT]) + "<br>";
        
        // AudioFormat
        if (_M.Data[(int)MovieField.AUDIOFORMAT] != "") __NewText += "<b>" + GetFieldName(MovieField.AUDIOFORMAT) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.AUDIOFORMAT]) + "<br>";

        //  Dvd region
        if (_M.Data[(int)MovieField.DVDREGION] != "") __NewText += "<b>" + GetFieldName(MovieField.DVDREGION) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.DVDREGION]) + "<br>";

        // Quality
        if (_M.Data[(int)MovieField.QUALITY] != "") __NewText += "<b>" + GetFieldName(MovieField.QUALITY) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.QUALITY]) + "<br>";


        // Subtitles
        if (_M.Data[(int)MovieField.SUBTITLES] != "") __NewText += "<b>" + GetFieldName(MovieField.SUBTITLES) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.SUBTITLES]) + "<br>";
        
        // Release
        if (_M.Data[(int)MovieField.RELEASE] != "") __NewText += "<b>" + GetFieldName(MovieField.RELEASE) + ":</b> " + Util.Strings.MarkupizeText(_M.Data[(int)MovieField.RELEASE]) + "<br>";
        
        __NewText += "<br>";

        // Description
        if (_M.Data[(int)MovieField.DESCRIPTION] != "") __NewText += "<b>" + GetFieldName(MovieField.DESCRIPTION) + ":</b> " + Util.Strings.MarkupizeTextBR(_M.Data[(int)MovieField.DESCRIPTION]);

        __NewText += "</body></html>";
        

        LoadFromString(__NewText);
        DisplayedMovie = _M;
    }



}
