/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using PluginLib;
using Gnome;
using System;
using System.Collections;
using MovieDatabase;
using System.Reflection;
using System.IO;

public class PluginManager
{
    public static ArrayList ImportPlugins;
    public static ArrayList ExportPlugins;
    public static ArrayList GetFromPlugins;

    private static void FindAssemblies (string dir)
    {

        if (dir == null || dir == "")
            return;
        
        DirectoryInfo info = new DirectoryInfo (dir);
        if (!info.Exists)
            return;
        
        foreach (FileInfo file in info.GetFiles ()) {
            if (file.Extension == ".dll") {
                try {
                    Assembly a = Assembly.LoadFrom (file.FullName);
                    ScanAssemblyForPlugins (a);
                    
                    } catch (Exception e) {
                        Console.WriteLine ("Error while scaning for plugins: " + file.Name + "('" + e.Message + "')");
                    }
            }
        }
    }
    
    /* Check, if a plugin is already loaded into memory
       to prevent duplication of plugins             */
    private static bool PluginAlreadyLoaded(Plugin _P)
    {
        foreach (Plugin __I in ImportPlugins)
            if (__I.GetName() == _P.GetName()) return true;

        foreach (Plugin __E in ExportPlugins)
            if (__E.GetName() == _P.GetName()) return true;
        
        foreach (Plugin __GF in GetFromPlugins)
            if (__GF.GetName() == _P.GetName()) return true;

        return false;
    }

    
    private static void ScanAssemblyForPlugins (Assembly assembly)
    {
        
        foreach (Type t in assembly.GetTypes ()) {
            /* Import */
            if ((t.IsSubclassOf (typeof (ImportPlugin)) && !t.IsAbstract)) {
                ImportPlugin plugin = (ImportPlugin) Activator.CreateInstance (t);
                plugin.Initialize ();
                if (!PluginAlreadyLoaded(plugin)) ImportPlugins.Add(plugin);
            }
            /* Export */
            if ((t.IsSubclassOf (typeof (ExportPlugin)) && !t.IsAbstract)) {
                ExportPlugin plugin = (ExportPlugin) Activator.CreateInstance (t);
                plugin.Initialize ();
                if (!PluginAlreadyLoaded(plugin)) ExportPlugins.Add(plugin);
            }
            /* Get From */
            if ((t.IsSubclassOf (typeof (GetFromPlugin)) && !t.IsAbstract)) {
                GetFromPlugin plugin = (GetFromPlugin) Activator.CreateInstance (t);
                plugin.Initialize ();
                if (!PluginAlreadyLoaded(plugin)) GetFromPlugins.Add(plugin);
            }
                            
        }

                
    }

    static PluginManager ()
    {
        ImportPlugins = new ArrayList();
        ExportPlugins = new ArrayList();
        GetFromPlugins = new ArrayList();
    

        // Local plugins have precedence over system plugins
        FindAssemblies(Gnome.User.DirGet() + "monotheka/Plugins/");
        FindAssemblies("Plugins/");
        FindAssemblies(Defines.PluginsPath);
        
    }
    
        
        
        
    

}
