/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;

namespace MovieDatabase
{
    public enum MovieField : int { ID = 0, FLAGS, NO, TITLE, TRANSLATEDTITLE, DIRECTOR, TIME, YEAR, COUNTRY, GENRE, 
                                       DISCS, VIDEOFORMAT, AUDIOFORMAT, QUALITY, SUBTITLES, RELEASE, BORROWER, 
                                       DESCRIPTION , ADDEDDATE, BORROWEDDATE, DVDREGION} ;

    public enum MovieFlag : int { LOST = 1, DAMAGED = 2, COPYNEEDED = 4, RARE = 8, FAVOURITE = 16, HARDDRIVE = 32, BORROWED = 64 };

    public class FlagsList {
        
        private ArrayList Flags;

        public FlagsList (string _FlagsString)
        {
            Flags = new ArrayList();

            if (_FlagsString != null && _FlagsString != "") {
                string[] _S = _FlagsString.Split(',');
                
                foreach (string _F in _S)
                    Flags.Add((MovieFlag)Convert.ToInt32(_F));
            }
        }

        public bool IsPresent(MovieFlag _F)
        {
            foreach (MovieFlag _MF in Flags)
                if (_MF == _F) return true;

            return false;
        }

        public override string ToString()
        {
            string __Ret = "";
            foreach (MovieFlag _MF in Flags)
                __Ret += Convert.ToString((int) _MF).PadLeft(3,'0') + ",";

            __Ret = __Ret.TrimEnd(',') ;
            return __Ret;
        }

        public void Add(MovieFlag _B)
        {
            // Check if we have it
            foreach (MovieFlag _MF in Flags)
                if (_MF == _B) return;
            
            
            Flags.Add((MovieFlag)Convert.ToInt32(_B));
            return;
        }

        public void Remove(MovieFlag _B)
        {
            // Check if we have it
            foreach (MovieFlag _MF in Flags)
                if (_MF == _B) {
                    Flags.Remove((MovieFlag)Convert.ToInt32(_B));
                    return;
                }
            return;
        }


        

    }

    public class Movie
    {
        public string[] Data;    
        
        /* Fill the data */
        private void FillData(string[] _Data)
        {
            if (_Data.Length > Data.Length) throw new Exception("Trying to create a Movie with too much fields!");
                        
            // Fill all the structures
            for (int __i = 0; __i < _Data.Length; __i++)
                Data[__i] = _Data[__i];         
            
        }

    
        /* Default Constructor */
        public Movie()
        {
            Data = new string[21];
            // Clear all 
            for (int __k = 0; __k < Data.Length; __k++)
                Data[__k] = null;
            
        }
            
        
        /* Create a copy of this object */
        public Movie Clone()
        {
            Movie _M = new Movie();
            for (int __k = 0; __k < Data.Length; __k++)
                _M.Data[__k] = Data[__k];
            return _M;
        }


        /* Quote all ' in array of strings */ 
        public void DBLize()
        {
            
            for (int __i = 0; __i < Data.Length; __i++)
                if (Data[__i] != null) Data[__i] = Data[__i].Replace("'", "''");
            
        }
 
        
        
        /* Removes useless spaces from start&end of string */
        public void TrimSpaces() 
        {
            
            for (int __i=0; __i < Data.Length; __i++) {
                if (Data[__i] != null) { 
                    Data[__i] = Data[__i].TrimStart(null);
                    Data[__i] = Data[__i].TrimEnd(null);
                }
            }
            
        }

    }


}
        
