/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using MovieDatabase;
using PluginLib;
using Gtk;
using Util;
using System.IO;

public class AVIGet : GetFromPlugin {
    
    private static readonly string ___Description = "Reads AVI RIFF file and detects the video & audio format/codec used.";
    private static readonly string ___SelectionDialog = "Please select the AVI file.";

    // GCONF
    private const string GConf_LastDirectory = "/apps/monotheka/plugins/AVIGet/LastDirectory" ;
    private const string GConf_LastDirectory_Default = null;


    public override int Initialize()
    {
        return 0;
    }

    public override string GetName() {
        return "Get from AVI";
    }

    public override string GetAuthor() {
        return "Michael Dominic K. <mdk@mdk.org.pl>";
    }

    public override string GetDescription() {
        return ___Description;
    }

    public override Movie Run(Movie _M) 
    {
        // Get the GConf key of last directory used
        string __LastDirectory = (string) GConfRegistry.Get(GConf_LastDirectory, GConf_LastDirectory_Default);

        // FIXME: What about the non-UTF8 paths????
        // Let's display the file selection dialog
        Util.Windows.FileSelectionDialogWindow __AVILocation = new Util.Windows.FileSelectionDialogWindow (__LastDirectory, ___SelectionDialog);

        try {
            if (__AVILocation.GetResult() != null) {
                // Set the GCONF Key
                GConfRegistry.Set (GConf_LastDirectory, Path.GetDirectoryName(__AVILocation.GetResult()) + "/");
                
                // RIFF stuff
                RIFFReader __RR = new RIFFReader(__AVILocation.GetResult());
                __RR.Run();
                return __RR.GetMovie();
            }
        } catch (RIFFReaderException __Excp) {
            new Util.Windows.OKDialogWindow(__Excp.Message , null, MessageType.Warning);
        }
    

        return null;
    }

}
