/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* REV: MDK
 * DESCRIPTION: Represents a html page containing a single movie
 * information from Filmweb
 */

using System;
using MovieDatabase;
using Util;

public class MoviePage {

    string HtmlData = null;
    string HtmlDescData = null;
    
    //////////////////////////////////////////////////
    
    /* Constructor */
    public MoviePage(string _Html)
    {
        Debug.Message("FilmwebGet.dll", "MoviePage", "Constructing...");
        HtmlData = _Html;

        string __PlotLink = Tags.AfterQueryInTag("alt=\"o filmie\"", "<a class=\"n\" href=\"", "\">", _Html);
        if (__PlotLink != null && __PlotLink != String.Empty) {
            Debug.Message("FilmwebGet.dll", "MoviePage", "Fetching additional HTML description data");
            HtmlDescData = Net.GetWebpageAsString(__PlotLink, "ISO-8859-2");
        }
    }

    
    /* Get Movie information */
    public Movie GetMovie()
    {
        Debug.Message("FilmwebGet.dll", "MoviePage", "Parsing HTML string data");

        // Translated title
        string __TranslatedTitle;
        try {
            string __TranslatedTitle_Temp = Strings.StartFromTill("<div class=\"tyt\">", "<", HtmlData);
            __TranslatedTitle = Strings.ClearString(__TranslatedTitle_Temp);
        } catch {
            __TranslatedTitle = null;
        }

        // Title
        string __Title;
        try {
            string __Title_Temp = Strings.StartFromTill("<span class=\"styt\">", "</span>", HtmlData);
            __Title_Temp = Strings.StripParenthesis(__Title_Temp);
            __Title = Strings.ClearString(__Title_Temp);
        } catch {
            __Title = null;
        }
        
        // Director
        string __Director;
        try {
            string __Director_Temp = Tags.AfterQueryInTag("yseria", "<a ", "</a>", HtmlData);
            __Director_Temp = Strings.StartFromTill(">" , "</a>", __Director_Temp);
            // FIXME: Strip (I)
            __Director = Strings.ClearString(__Director_Temp);
        } catch {
            __Director = null;
        }

        // Genre
        string __Genre;
        try {
            string __Genre_Temp = Tags.AfterQueryInTag("gatunek:", "<b>", "</b>", HtmlData);
            __Genre_Temp = Strings.StripMarkup(__Genre_Temp);
            __Genre = Strings.ClearString(__Genre_Temp);
        } catch {
            __Genre = null;
        }

        // Production
        string __Production;
        try {
            // FIXME: CelarString should also fix multiple spaces
            string __Production_Temp = Tags.AfterQueryInTag("produkcja:", "<b>", "</b>", HtmlData);
            __Production_Temp = Strings.StripMarkup(__Production_Temp);
            __Production = Strings.ClearString(__Production_Temp);
        } catch {
            __Production = null;
        }
        
        // Runtime
        string __Runtime;
        try {
            string __Runtime_Temp = Strings.StartFromTill("czas trwania:", "\n", HtmlData);
            __Runtime = Strings.ClearString(__Runtime_Temp);
        } catch {
            __Runtime = null;
        }

        
        // Year
        string __Year;
        try {
            string __Year_Temp = Strings.StartFromTill("<div class=\"tyt\">", "</div>", HtmlData);
            string[] __Tags = Tags.SplitTags("<span class=\"styt\">", "</span>", __Year_Temp);
            string[] __Tags2 = Tags.SplitTags("(", ")", __Tags[__Tags.Length-1]);
            __Year = Strings.ClearString(__Tags2[__Tags2.Length-1]);
        } catch {
            __Year = null;
        }

        // Description
        string __Description;
        
        try {
            if (HtmlDescData != null) {
                
                string __Description_Temp = Tags.AfterQueryInTag("<img src=\"http://gfx.filmweb.pl/gf/he/g_bg/o.gif\" width=\"49\" height=\"9\" alt=\"opisy\" />", "<div align=\"justify\">", "</div>", HtmlDescData);
                __Description_Temp = Strings.StripMarkup(__Description_Temp);
                __Description_Temp = Strings.ClearString(__Description_Temp);
                __Description = Strings.NoNewLines(__Description_Temp);
            } else {
                
                // Try to get a 'short' description.
                string __Description_Temp = Strings.StartFromTill("<div style=\"margin-bottom: 5px;\"><img src=\"http://gfx.filmweb.pl/gf/he/g_bg/of.gif\" width=\"70\" height=\"9\" alt=\"o filmie\"/></div>", "</div>", HtmlData);
                __Description_Temp = Strings.StripMarkup(__Description_Temp);
                __Description_Temp = Strings.ClearString(__Description_Temp);
                __Description = Strings.NoNewLines(__Description_Temp);
            }
                
        } catch {
            __Description = null;
        }

        // Fill in
        Movie __M = new Movie();

        if ( __TranslatedTitle != null && __TranslatedTitle != String.Empty && ( __Title == null || __Title == String.Empty)) {
            __Title = __TranslatedTitle;
            __TranslatedTitle = null;
        }
                     
        __M.Data[(int)MovieField.TRANSLATEDTITLE] = __TranslatedTitle;
        __M.Data[(int)MovieField.TITLE] = __Title;
        
        __M.Data[(int)MovieField.DIRECTOR] = __Director;
        __M.Data[(int)MovieField.GENRE] = __Genre;
        __M.Data[(int)MovieField.TIME] = __Runtime;
        __M.Data[(int)MovieField.COUNTRY] = __Production;
        __M.Data[(int)MovieField.DESCRIPTION] = __Description;
        __M.Data[(int)MovieField.YEAR] = __Year;

        // Debug
        // FIXME: Some debug stuff
             
        return __M;
    }

}
